\section[IORef]{Mutable Variables}

\begin{code}
module IORef
	( IORef		      -- abtract, instance of: Eq
        , newIORef	      -- :: a -> IO (IORef a)
        , readIORef	      -- :: IORef a -> IO a
        , writeIORef	      -- :: IORef a -> a -> IO ()
	, modifyIORef	      -- :: IORef a -> (a -> a) -> IO ()
	, updateIORef	      -- deprecated, use modifyIORef

#if !defined(__PARALLEL_HASKELL__) && !defined(__HUGS__)
	, mkWeakIORef           -- :: IORef a -> IO () -> IO (Weak (IORef a))
#endif
	) where

#ifndef __HUGS__
import PrelGHC		( RealWorld, mkWeak# )
import PrelArr		( STRef(..) )
#if !defined(__PARALLEL_HASKELL__)
import PrelIOBase	( IO(..) )
import PrelWeak
#endif
#else 
import PrelPrim ( RealWorld, IORef, newIORef, writeIORef, readIORef )
#endif /* ! __HUGS__ */
import ST		( stToIO, newSTRef, readSTRef, writeSTRef )
\end{code}

\begin{code}
#ifdef __HUGS__
-- IORef, newIORef, readIORef, writeIORef can be found
-- in the prelude
#else

newtype IORef a = IORef (STRef RealWorld a) deriving Eq

newIORef    :: a -> IO (IORef a)
newIORef v = stToIO (newSTRef v) >>= \ var -> return (IORef var)

readIORef   :: IORef a -> IO a
readIORef  (IORef var) = stToIO (readSTRef var)

writeIORef  :: IORef a -> a -> IO ()
writeIORef (IORef var) v = stToIO (writeSTRef var v)

#ifndef __PARALLEL_HASKELL__
mkWeakIORef :: IORef a -> IO () -> IO (Weak (IORef a))
mkWeakIORef r@(IORef (STRef r#)) f = IO $ \s ->
  case mkWeak# r# r f s of (# s1, w #) -> (# s1, Weak w #)
#endif

#endif

modifyIORef :: IORef a -> (a -> a) -> IO ()
modifyIORef ref f = writeIORef ref . f =<< readIORef ref

-- deprecated, use modifyIORef
updateIORef :: IORef a -> (a -> a) -> IO ()
updateIORef = modifyIORef
\end{code}
